from linktune.api.tidal import Tidal
from linktune.api.spotify import Spotify
from linktune.api.deezer import Deezer
from linktune.api.applemusic import AppleMusic
from linktune.api.youtube import YouTube
from linktune.config.config import SPOTIPY_CLIENT_ID, SPOTIPY_CLIENT_SECRET

class Convert:
    def __init__(self):
        self.spotify = Spotify(SPOTIPY_CLIENT_ID, SPOTIPY_CLIENT_SECRET)
        self.tidal = Tidal()
        self.deezer = Deezer()
        self.apple = AppleMusic()
        self.youtube = YouTube()

    def convert_link(self, link, target_service):
        if "spotify" in link:
            source_service = self.spotify
        elif "tidal" in link:
            source_service = self.tidal
        elif "deezer" in link:
            source_service = self.deezer
        elif 'apple' in link:
            source_service = self.apple
        elif 'youtube' in link:
            source_service = self.youtube
        else:
            return None

        track_info = source_service.get_track_info(link)

        if not track_info:
            return None

# TODO: clean up this repetitive crap
        if target_service == 'spotify':
            return self.spotify.get_url(track_info)
        elif target_service == 'tidal':
            return self.tidal.get_url(track_info)
        elif target_service == 'apple':
            return self.apple.get_url(track_info)
        elif target_service == 'youtube':
            return self.youtube.get_url(track_info)
        else:
            return "Something went wrong."


from tidal_unofficial import TidalUnofficial


class Tidal:
    def __init__(self):
        self.tidal = TidalUnofficial({'user_agent': 'my_user_agent'})

    def get_track_info(self, track_url):
        track_id = self._get_track_id(track_url)
        if not track_id:
            return None
        track = self.tidal.get_track(track_id)
        title = track['title']
        artist = track['artist']['name']

        return {'artist': artist, 'title': title}

    def _get_track_id(self, track_url):
        track_id = None
        track_id = track_url.split('/')[-1]
        return track_id

    def get_url(self, info):
        title = info['title']
        if isinstance(info['artist'], list):
            artist = info['artist'][0]
        else:
            artist = info['artist']
        query = f"{title} {artist}"

        result = self.tidal.search(query, search_type='tracks', limit=1)
        return f"info: {title} by {artist}, url: {result['items'][0]['url']}"


from setuptools import setup, find_packages
from pip._internal.req import parse_requirements

# Parse the requirements.txt file
requirements = parse_requirements('requirements.txt', session='hack')

setup(
    name='linktune',
    version='0.1',
    install_requires=[str(req.requirement) for req in requirements],
    packages=find_packages(),
    py_modules=['api', 'cli', 'config'],
    entry_points={
    'console_scripts': [
        'linktune = cli.main:main'
        ]
    }
)


from linktune.api.convert import Convert
from linktune.api.search import search_track

def convert(args):
    # get target service and song link from arguments
    service, link = args.target, args.link

    # call convert_link function from convert api
    converted_link = Convert().convert_link(link, service)
    if converted_link is None:
        print(f"Unable to find information for {link} on {service}")
    print(converted_link)

def search(args):
    artist, title, service = args.artist, args.title, args.service

    print(search_track(artist, title, service))


import requests

class AppleMusic:

    itunes_base = 'https://itunes.apple.com/'

    def get_track_info(self, track_url):
        track_id = self._get_track_id(track_url)

        query = f'{self.itunes_base}lookup?id={track_id}'

        res = requests.get(query)
        res.raise_for_status()

        # get results dict from the json response
        data = res.json()['results'][0]

        title = data['trackName']
        artist = data['artistName']
        return {'artist': artist, 'title': title}

    def _get_track_id(self, track_url):
        track_id = track_url.split('?i=')[1]
        if not track_id:
            return "Could not locate track id"
        return track_id
    
    def get_url(self, info):
        title = info['title']
        if isinstance(info['artist'], list):
            artist = info['artist'][0]
        else:
            artist = info['artist']

        query = f'{self.itunes_base}search?term={artist}+{title}'

        res = requests.get(query)
        res.raise_for_status()

        # get json response of matching tracks
        data = res.json()['results'][0]

        link, artist, title = data['trackViewUrl'], data['artistName'], data['trackName']

        return f'{title} ' 'by ' f'{artist}'': ' f'{link}'


import argparse
from linktune.cli.commands import convert, search

def create_parser():
    parser = argparse.ArgumentParser(description="LinkTune: Convert music links from one service to another and search for tracks across services.")
    subparsers = parser.add_subparsers(help='Commands', dest='command')

    # parser for convert command
    convert_parser = subparsers.add_parser('convert', help='Convert a music link to that of another music service.')
    convert_parser.add_argument('link', help='The link to be converted.')
    convert_parser.add_argument('target', help='The target music service to convert to.')
    convert_parser.set_defaults(func=convert)

    # parser for search command
    search_parser = subparsers.add_parser('search', help='Search for a track by artist and title on a music service.')
    search_parser.add_argument('--artist', '-a', help='Artist name')
    search_parser.add_argument('--title', '-t', help='Track title')
    search_parser.add_argument('--service', '-s', choices=['spotify', 'tidal', 'deezer', 'apple', 'all'], default='all', help='The music service to search.')
    search_parser.set_defaults(func=search)

    return parser


from flask import Flask, request, render_template
from linktune.api.convert import Convert

app = Flask(__name__, static_folder='static')

# Set up route handlers
@app.route('/')
def index():
    return app.send_static_file('index.html')

@app.route('/convert')
def convert():
    print('convert request received')
    url = request.args.get('url')
    target_service = request.args.get('target_service')
    return Convert().convert_link(url, target_service)

if __name__ == '__main__':
    app.run(host='0.0.0.0', port=5000, debug=True)


from linktune.api.tidal import Tidal
from linktune.api.spotify import Spotify
from linktune.api.deezer import Deezer
from linktune.api.applemusic import AppleMusic
from linktune.config.config import SPOTIPY_CLIENT_ID, SPOTIPY_CLIENT_SECRET

# TODO: implement map of services for quicker look up; add "all" option which is the default
def search_track(artist, title, service='all'):
    info = {'title': title, 'artist': artist}

    api_map = {
        'spotify': (Spotify, SPOTIPY_CLIENT_ID, SPOTIPY_CLIENT_SECRET),
        'tidal': (Tidal,),
        'deezer': (Deezer,),
        'apple': (AppleMusic,),
    }

    if service == 'all':
        results = []
        for service in api_map:
            api_args = api_map[service]
            api_class = api_args[0]
            api = api_class(*api_args[1:])
            results.append(f"{service}: {api.get_url(info)}")
        return results

    if service in api_map:
        api_args = api_map[service]
        api_class = api_args[0]
        api = api_class(*api_args[1:])
        return api.get_url(info)
    else:
        return "Service not supported"




import requests

class Deezer:
    base_url = 'https://api.deezer.com/'

    def get_track_info(self, track_url):
        print('Searching for track...')
        track_id = self._get_track_id(track_url)

        query = f'{self.base_url}/track/{track_id}'
        response = requests.get(query)
        response.raise_for_status()

        data = response.json()

        title = data['title']
        artist = data['artist']['name']

        return {'artist': artist, 'title': title}

    def _get_track_id(self, track_url):
        _, _, track_id = track_url.rpartition('/')
        print(f'Track id is {track_id}')
        if not track_id:
            return "Could not locate track id"
        return track_id
    
    def get_url(self, info):
        title = info['title']
        artist = info['artist']

        query = f'{self.base_url}/search?q=artist:"{artist}" track:"{title}"'
        response = requests.get(query)
        response.raise_for_status()

        data = response.json()['data'][0]
        
        link = data['link']

        return f"info: {title} by {artist}, url: {link}"



import spotipy
import re
from spotipy.oauth2 import SpotifyClientCredentials
from spotipy.cache_handler import MemoryCacheHandler

class Spotify:
    def __init__(self, client_id, client_secret):
        self.sp = spotipy.Spotify(auth_manager=SpotifyClientCredentials(client_id=client_id, client_secret=client_secret, cache_handler=MemoryCacheHandler()))

    def get_track_info(self, track_url):
        print('Searching for track...')
        track_id = self._get_track_id(track_url)
        if not track_id:
            return 'Could not identify Spotify track ID'

        track = self.sp.track(track_id)
        artists = [artist['name'] for artist in track['artists']]
        title = track['name']

        return {'artist': artists, 'title': title}

    def _get_track_id(self, track_url):
        track_id = None
        # check if Spotify internal id is appended to the link and remove if so
        if '?si=' in track_url:
            track_url = track_url.split('?si=')[0]
        if 'open.spotify.com/track/' in track_url:
            _, _, track_id = track_url.rpartition('/')
        elif 'spotify:track:' in track_url:
            _, _, track_id = track_url.rpartition(':')

        return track_id

    def get_url(self, info):
        title = info['title']
        # remove extra characters present in track title
        title = re.sub("\(.*?\)|\[.*?\]","",title).rstrip()

        artist = info['artist']
        query = f"{title} artist:{artist}"

        result = self.sp.search(query, limit=1, type='track')

        if result['tracks']['total'] > 0:
            uri = result['tracks']['items'][0]['uri']
            _, _, track_id = uri.rpartition(':')
            return f"info: {title} by {artist}, url: open.spotify.com/track/{track_id}"
        else:
            return 'Track not found.'


from ytmusicapi import YTMusic
import re

# this can convert links to both YouTube Music and regular YouTube links
# as long as the id and externalID are the same...

class YouTube:
    def __init__(self):
        self.youtube = YTMusic()

    def get_track_info(self, track_url):
        print('Searching for track...')
        track_id = self._get_track_id(track_url)
        if not track_id:
            return 'Could not identify YouTube Music track ID from url.'

        track = self.youtube.get_song(track_id)
        # returns dictionary with song metadata
        artist = track['videoDetails']['author']
        title = track['videoDetails']['title']

        # TODO: implement integration with regular YouTube
        # external_video_id = track['microformat']

        return {'artist': artist, 'title': title}

# https://music.youtube.com/watch?v=tqxRidAWER8
    
    def _get_track_id(self, track_url):
        track_id = None
        track_id = re.search(r"v=([\w-]+)", track_url)
        track_id = track_id.group(1)

        return track_id
    
    def get_url(self, info):
        title = info['title']
        if isinstance(info['artist'], list):
            artist = info['artist'][0]
        else:
            artist = info['artist']

        # store top result of search
        res = self.youtube.search(f"{artist} {title}", filter = 'songs')[0]
        track_id = res['videoId']
        title = res['title']
        # TODO: edit artist to actually find all artist names in the array but this works for now
        artist = res['artists'][0]['name']

        return(f"info: {title} by {artist}: url: https://music.youtube.com/watch?v={track_id}")

# testing using regular youtube links from unofficial accounts... actually does work but really wouldnt be that great for non-verified artists
# I could make it take the track title from youtube and use that as the search query for other services?
# id = 'xAok29V8Bf4'
# yt = YouTube()
# print(yt.get_track_info('https://music.youtube.com/watch?v=xAok29V8Bf4'))


from linktune.cli.arguments import create_parser

def main():
    parser = create_parser()

    args = parser.parse_args()

    if not args.command:
        parser.print_help()
        return

    args.func(args)

if __name__ == '__main__':
    main()